import unittest
import tensorflow as tf
import numpy as np
from zetane import KerasViz
from zetane import context as ztn

class TestKeras(unittest.TestCase):
    def setUp(self):
        self.viz = KerasViz()
        self.engine = ztn.Context()
        self.spec = (tf.TensorSpec(shape=(None,28,28,1), dtype=tf.float32, name="input"),)
        self.random_input = np.random.randn(1, 28, 28, 1)

        num_classes = 10
        self.model = tf.keras.Sequential(
            [
                tf.keras.layers.Input(shape=(28,28,1), dtype=tf.float32),
                tf.keras.layers.Conv2D(32, kernel_size=(3, 3), activation="relu"),
                tf.keras.layers.MaxPooling2D(pool_size=(2, 2)),
                tf.keras.layers.Conv2D(64, kernel_size=(3, 3), activation="relu"),
                tf.keras.layers.MaxPooling2D(pool_size=(2, 2)),
                tf.keras.layers.Flatten(),
                tf.keras.layers.Dropout(0.5),
                tf.keras.layers.Dense(num_classes, activation="softmax"),
            ]
        )
        self.expected_y = self.model.predict(self.random_input)

    def test_viz(self):
        self.viz.create_keras_model(self.model, self.spec)
        self.viz.model_debug(self.random_input)

        output_data = self.engine.form('graph.nodes.node=Softmax.outputs.output.tensor.data').update()
        output_shape = self.engine.form('graph.nodes.node=Softmax.outputs.output.tensor.shape').update()

        data_vals = output_data.get_values()
        shape_vals = output_shape.get_values()

        np.testing.assert_array_almost_equal(data_vals, self.expected_y.flatten())

    def test_path(self):
        import tf2onnx
        output_path = 'keras2_model.onnx'
        keras_model_proto, _ = tf2onnx.convert.from_keras(
            self.model, self.spec, output_path=output_path)
        model = self.engine.model()
        model.model(output_path).update()
        model.inputs(self.random_input).debug()

        output_data = self.engine.form('graph.nodes.node=Softmax.outputs.output.tensor.data').update()
        output_shape = self.engine.form('graph.nodes.node=Softmax.outputs.output.tensor.shape').update()

        data_vals = output_data.get_values()
        shape_vals = output_shape.get_values()

        np.testing.assert_array_almost_equal(data_vals, self.expected_y.flatten())
