import unittest

class TestCharts(unittest.TestCase):

    # Only use setUp() and tearDown() if necessary

    def setUp(self):
        self.zcontext = ztn.Context()

        self.zcontext.launch()
        self.target_updates_per_second = 5.0
        self.framecount = 0

    def tearDown(self):
        self.zcontext.clear_universe()
        self.framecount = 0

    def test_bar_chart(self):
        zbchart = self.zcontext.chartDEPRECATED()  # enables auto update()
        zbchart.color(0.6, .5, .7)
        zbchart.position(-20, 0, -10)
        zbchart.as_bar_graph()
        labels_under_bar_graph = {'Model A': 0, 'Model B': 1, 'Model C': 2, 'Model D': 3, 'Model F': 4}
        zbchart.set_labels_with_indices(labels_under_bar_graph)
        self.framecount = 0

        # Main loop
        while self.framecount < 10:
            bdata_points = [x * 2.4 for x in abs(randn(5)).tolist()]
            zbchart.points(bdata_points)
            zbchart.update()
            time.sleep(1.0 / self.target_updates_per_second)

            self.framecount += 1

        zform = self.zcontext.form("chart", get_child_data=True).update()
        self.assertTrue(zform.has_received())

    def test_chart(self):
        # Example chart
        zchart = self.zcontext.chartDEPRECATED()  # enables auto update()
        i = 2
        self.framecount = 0

        # Main loop
        while self.framecount < 10:
            data_points = [x * 1.2 for x in randn(i).tolist()]
            zchart.points(data_points)
            zchart.color(0.6, .5, .7)
            zchart.position(-20, 0, -10)
            zchart.smooth()
            zchart.compress()
            i += 1
            zchart.update()

            time.sleep(1.0 / self.target_updates_per_second)
            self.framecount += 1

        zform = self.zcontext.form("chart", get_child_data=True).update()
        self.assertTrue(zform.has_received())

    def test_chart3D(self):

        # Example chart
        zchart = self.zcontext.chart3D()  # enables auto update()
        zchart.position(0, 0, -10)
        zchart.scale(10, 1, 10)
        zchart.wireframe()
        data = np.load("data/arr_3d.npy")
        zchart.set_3Dpoints(data)
        zchart.update()

        zform = self.zcontext.form("chart3D", get_child_data=True).update()
        self.assertTrue(zform.has_received())

    def test_chart_Pie(self):
        zPieChart = self.zcontext.pie_chart()
        zPieChart.position(0, 0, -5)
        label_list = ['Eyy', 'Bois', 'We', 'Did', 'The', 'Thing']
        # zPieChart.pielabels(label_list)
        self.framecount = 0

        while self.framecount < 10:
            data_points = [x * 1.2 for x in randn(6).tolist()]
            labeled_points = {}
            counter = 0
            for label in label_list:
                labeled_points[label] = data_points[counter]
                counter += 1
            # zPieChart.pielabels(label_list)
            zPieChart.pielabels(labeled_points)
            zPieChart.update()

            time.sleep(1.0 / self.target_updates_per_second)
            self.framecount += 1

        zform = self.zcontext.form("chart", get_child_data=True).update()
        self.assertTrue(zform.has_received())

    def test_filled_chart(self):
        # Example chart
        zchart = self.zcontext.chartDEPRECATED()  # enables auto update()
        zchart.color(0.6, .5, .7)
        zchart.position(-20, 0, -10)
        # zchart.smooth()
        # zchart.heightmap()
        # zchart.wireframe()
        # zchart.compress()
        amount_of_points = 2
        self.framecount = 0

        # Main loop
        while self.framecount < 10:
            # data_points = [x * 1.2 for x in randn(amount_of_points).tolist()]
            # amount_of_points+=1
            data_points = [x * 1.2 for x in randn(400).tolist()]
            zchart.points(data_points)
            zchart.add_border()
            zchart.update()

            time.sleep(1.0 / self.target_updates_per_second)

            self.framecount += 1

        zform = self.zcontext.form("chart", get_child_data=True).update()
        self.assertTrue(zform.has_received())

    def test_chart_perlin_noise(self):

        # Example chart
        zchart = self.zcontext.chartDEPRECATED()  # enables auto update()
        zchart.color(0.6, .5, .7)
        zchart.position(-20, 0, -10)
        #zchart.heightmap()

        shape = (1024, 1024)
        scale = 100.0
        octaves = 6
        persistence = 0.5
        lacunarity = 2.0

        # Main loop
        start = 0

        while start < 10:
            data_points = [noise.pnoise1(i / scale) for i in range(start, start + 400)]
            zchart.points(data_points)
            zchart.update()
            start += 1

            time.sleep(1.0 / self.target_updates_per_second)

            self.framecount += 1

        zform = self.zcontext.form("chart", get_child_data=True).update()
        self.assertTrue(zform.has_received())


class TestDial(unittest.TestCase):

    def test_dial(self):
        zcontext = ztn.Context().launch()
        zcontext.connect()
        target_updates_per_second = 60.0
        framecount = 0

        zDial = zcontext.dial()

        while framecount < 10:
            zDial.set_max(10)
            zDial.set_indicator_value(framecount % 11)
            zDial.update()
            time.sleep(1.0 / target_updates_per_second)
            framecount += 1

        zform = zcontext.form("dial", get_child_data=True).update()
        self.assertTrue(zform.has_received())

        zcontext.clear_universe()


class TestNumpy(unittest.TestCase):

    def setUp(self):
        self.zcontext = ztn.Context()
        self.zcontext.launch()

    def tearDown(self):
        self.zcontext.clear_universe()
        self.framecount = 0

    def test_1D(self):
        npy1D = np.random.rand(15)
        znpy1D = self.zcontext.numpy(npy1D)
        znpy1D.update()

        zform = self.zcontext.form("numpy", get_child_data=True).update()
        self.assertTrue(zform.has_received())

    def test_2D(self):
        npy2D = np.random.rand(100, 100)
        znpy2D = self.zcontext.numpy(npy2D)
        znpy2D.position(y=2)
        znpy2D.update()

        zform = self.zcontext.form("numpy", get_child_data=True).update()
        self.assertTrue(zform.has_received())

    def test_3D(self):
        npy3D = np.random.rand(5, 5, 5)
        znpy3D = self.zcontext.numpy(npy3D)
        znpy3D.position(y=-2)
        znpy3D.update()

        zform = self.zcontext.form("numpy", get_child_data=True).update()
        self.assertTrue(zform.has_received())

    def test_4D(self):
        npy4D = np.random.rand(12, 12, 12, 12)
        znpy4D = self.zcontext.numpy(npy4D)
        znpy4D.position(x=4)
        znpy4D.update()

        zform = self.zcontext.form("numpy", get_child_data=True).update()
        self.assertTrue(zform.has_received())


class TestForm(unittest.TestCase):

    def test_form(self):
        zcontext = ztn.Context()
        zcontext.launch()

        ztxt = zcontext.text("hello world")
        ztxt.position(1, 2, 0).update()

        zform = zcontext.form("text", get_child_data=True).update()
        self.assertTrue(zform.has_received())

        zcontext.clear_universe()


class TestText(unittest.TestCase):

    def setUp(self):
        # Launch zetane session
        self.zcontext = ztn.Context()
        self.zcontext.launch()

    def tearDown(self):
        self.zcontext.clear_universe()
        self.framecount = 0

    def test_text_modifiers(self):
        ztxt = self.zcontext.text("hello world")
        ztxt.position(1, 1, 1) \
            .scale(5, 5, 5) \
            .font('roboto-mono') \
            .font_size(.5) \
            .prefix('$') \
            .postfix('#') \
            .precision(5) \
            .fixed(True) \
            .chars_per_line(30) \
            .billboard(True) \
            .gradient(color_list=((0.1, 0.2, 0.3), (0.4, 0.8, 0.4), (1, 1, 1))) \
            .color((1, 0, 1)) \
            .highlight((0.5, 0.5, 1, 0.1)) \
            .update()

        zform = self.zcontext.form("text", get_child_data=True).update()
        self.assertTrue(zform.has_received())

    def test_pre_postfix(self):
        ztxt = self.zcontext.text(3)
        # ztxt2 = zcontext.text("Numba")
        for i in range(1, 100):
            print(i)
            time.sleep(0.08)
            ztxt.position(i / 100.).color((.5, math.cos(i), math.sin(5 * i))).font_size(0.2).update()
            # Cool stuff for no reason
            ztxt.gradient(color_list=((math.sin(0.1 * i), math.sin(0.2 * i), math.sin(0.3 * i)),
                                      (math.cos(0.4 * i), math.cos(0.8 * i), 0.4),
                                      (math.sin(i), math.cos(i), math.sin(i))))
            ztxt.scale(math.cos(i),
                       math.sin(5 * i))  # Currently seems text doesn't scale - remove this comment when it does
            if i % 10 == 0:
                # Load a random prefix and postfix
                ztxt.prefix(random.choice(string.punctuation))
                ztxt.postfix(random.choice(string.punctuation))
            ztxt.update()

        zform = self.zcontext.form("text", get_child_data=True).update()
        self.assertTrue(zform.has_received())


class TestImage(unittest.TestCase):

    def setUp(self):
        # Launch zetane session
        self.zcontext = ztn.Context()
        self.zcontext.launch()

    def tearDown(self):
        self.zcontext.clear_universe()
        self.framecount = 0

    def test_image(self):
        images_dir = "../../z10/images/VggImage"

        progress = self.zcontext.text("Loading images from " + images_dir)\
            .font_size(.4).chars_per_line(100).position(-5, -4, 0).update()
        zimage = self.zcontext.image().rotation(0, 1.56, 0).position(7.7, -3.6, -2)
        total_size = 0

        # Load images from dir
        before = time.perf_counter()
        for file in os.listdir(images_dir):
            imagefile = images_dir + '/' + file
            zimage.update(filepath=imagefile)
            total_size += os.path.getsize(imagefile)
            progress.text(f"Loaded {total_size / 1000000.0:.2f}MB").update()
        after = time.perf_counter()
        progress.text(f"Loaded {total_size / 1000000.0:.2f}MB in {after - before:0.2f} seconds").update()

        zform = self.zcontext.form("image_wrapper", get_child_data=True).update()
        self.assertTrue(zform.has_received())

    def test_highlight(self):
        imagefile = "z10/images/VggImage/catimage2.png"
        progress = self.zcontext.text("CAT THE RAINBOW").font_size(.4).position(-5, -4, 0).update()

        # Cat the rainbow
        zred__ = self.zcontext.image(filepath=imagefile).rotation(0, 1.56, 0).position(-5, -3.6, -2)
        zblue_ = self.zcontext.image(filepath=imagefile).rotation(0, 1.56, 0).position(+0, -3.6, -2)
        zgreen = self.zcontext.image(filepath=imagefile).rotation(0, 1.56, 0).position(+5, -3.6, -2)
        for i in range(10):
            zred__.highlight(rgb=(1, 0, 0), amount=math.sin(i / 1000.0) / 2).update()
            zblue_.highlight(rgb=(0, 1, 0), amount=math.sin(i / 2000.0) / 2).update()
            zgreen.highlight(rgb=(0, 0, 1), amount=math.sin(i / 3000.0) / 2).update()
        progress.text("CAT THE RAINBOW!").update()

        zform = self.zcontext.form("text", get_child_data=True).update()
        self.assertTrue(zform.has_received())


class TestMesh(unittest.TestCase):

    def test_mesh(self):
        # Launch zetane session
        zcontext = ztn.Context().launch()
        target_updates_per_second = 10.0
        framecount = 0

        # Example mesh
        mesh_dir = "/data/Male_figure_16k.obj"
        zmesh = zcontext.mesh(mesh_dir)
        zmesh.position(z=-5)
        zmesh.backface_culling(True)
        zmesh.update()

        # Update loop
        i = 0.0
        show_wireframe = True
        while i < 10:
            zmesh.highlight(r=.5, g=0, b=.2, a=math.sin(i / 2500.0))
            zmesh.transparency(math.sin(i / 5000.0))
            zmesh.wireframe(show_wireframe)
            zmesh.update()
            i += 1
            if (i % 5000 == 0):
                show_wireframe = not show_wireframe

        zform = zcontext.form("mesh", get_child_data=True).update()
        self.assertTrue(zform.has_received())

        zcontext.clear_universe()


class TestPointcloud(unittest.TestCase):

    def setUp(self):
        # Launch zetane session
        self.zcontext = ztn.Context()
        self.zcontext.launch()
        self.target_updates_per_second = 10.0
        self.framecount = 0

    def tearDown(self):
        self.zcontext.clear_universe()
        self.framecount = 0

    def test_pointcloud_from_obj(self):
        # Example mesh
        obj_dir = "/data/Male_figure_16k.obj"
        zpointcloud = self.zcontext.pointcloud(obj_dir)
        zpointcloud.position(z=-5)
        zpointcloud.update()

        # Update loop
        i = 0.0
        while i < 10:
            zpointcloud.point_is_deep(math.sin(i / 2500.0) > 0)
            zpointcloud.point_is_circle(math.sin(i / 5000.0) > 0)
            zpointcloud.point_size(math.fabs(math.sin(i / 2500.0)))
            zpointcloud.update()
            i += 1

        zform = self.zcontext.form("pointcloud", get_child_data=True).update()
        self.assertTrue(zform.has_received())
