import unittest
import numpy as np
import time
import os
from numpy.random import randn
from zetane import context as ztn

class TestGraphique(unittest.TestCase):

    def setUp(self):
        self.zcontext = ztn.Context()
        self.zcontext.clear_universe()
        self.target_updates_per_second = 5.0
        self.framecount = 0

    def tearDown(self):
        self.zcontext.clear_universe()
        self.zcontext.disconnect()
        self.framecount = 0

    @unittest.skip("this data file doesn't exist")
    def test_3d_chart(self):
        # Example chart
        zchart = self.zcontext.chart('Surface chart', domain=[0.0, 0.0], range=[0.0, 10.0])
        metric_1 = zchart.metric(label='Metric 1')
        data_dir = os.path.abspath(os.path.join(currdir, '../data/'))
        data_file = os.path.join(data_dir, "arr_3d.npy")
        data = np.load(data_file)
        metric_1.set_3Dpoints(data)
        zchart.set_type('Surface')
        zchart.include_metrics([metric_1])
        zchart.update()

        zform = self.zcontext.form("surface_chart", get_child_data=True).update()
        self.assertTrue(zform.has_received())

    def test_bar_chart(self):

        # Example chart
        zchart = self.zcontext.chart('Bar chart', domain=[0.0, 0.0], range=[0.0, 10.0])

        metric_1 = zchart.metric(label='Metric 1')
        metric_2 = zchart.metric(label='Metric 2')
        metric_3 = zchart.metric(label='Metric 3')
        zchart.set_type('Bar')
        zchart.position(-5, 0, 5)

        # Main loop
        while self.framecount < 5:
            data_points_1 = [x * 1.2 for x in abs(randn(10)).tolist()]
            data_points_2 = [x * 3.2 for x in abs(randn(10)).tolist()]
            data_points_3 = [x * 3.2 for x in abs(randn(10)).tolist()]
            metric_1.append_values(y=data_points_1)
            metric_2.append_values(y=data_points_2)
            metric_3.append_values(y=data_points_3)
            zchart.include_metrics([metric_1, metric_2, metric_3])
            zchart.update()
            time.sleep(1.0 / self.target_updates_per_second)
            self.framecount += 1

        zchart.debug()
        zform = self.zcontext.form("bar_chart", get_child_data=True).update()
        self.assertTrue(zform.has_received())

    def test_chart(self):
        # Example chart
        zchart = self.zcontext.chart('Line chart', domain=[0.0, 0.0], range=[0.0, 10.0])

        metric_1 = zchart.metric(label='Metric 1')
        metric_2 = zchart.metric(label='Metric 2')
        metric_3 = zchart.metric(label='Metric 3')
        zchart.set_type('Line')
        zchart.set_attribute(['compressed'])
        zchart.position(-10, -5, 5)
        n = 10

        # Main loop
        while self.framecount < 5:
            data_points_1 = [x * 1.2 for x in randn(n).tolist()]
            data_points_2 = [x * 3.2 for x in randn(n).tolist()]
            data_points_3 = [x * 3.2 for x in randn(n).tolist()]
            metric_1.append_values(y=data_points_1)
            metric_2.append_values(y=data_points_2)
            metric_3.append_values(y=data_points_3)
            zchart.include_metrics([metric_1])
            zchart.update()
            time.sleep(1.0 / self.target_updates_per_second)
            self.framecount += 1

        zform = self.zcontext.form("line_chart", get_child_data=True).update()
        self.assertTrue(zform.has_received())

    def test_dial(self):
        # Example chart
        zchart = self.zcontext.chart('Dial', domain=[0.0, 0.0], range=[0.0, 10.0])

        metric_1 = zchart.metric(label='Metric 1')
        metric_2 = zchart.metric(label='Metric 2')
        metric_3 = zchart.metric(label='Metric 3')
        zchart.set_type('Dial')
        zchart.position(2, 2, 5)

        # Main loop
        while self.framecount < 5:
            data_points_1 = [x * 10 for x in abs(randn(10)).tolist()]
            data_points_2 = [x * 3.2 for x in abs(randn(10)).tolist()]
            data_points_3 = [x * 3.2 for x in abs(randn(10)).tolist()]
            metric_1.append_values(
                y=data_points_1)  # Only the last point of this metric is displayed (in the case of dials)
            metric_2.append_values(y=data_points_2)
            metric_3.append_values(y=data_points_3)
            zchart.include_metrics([metric_1, metric_2, metric_3])
            zchart.update()
            time.sleep(1.0 / self.target_updates_per_second)
            self.framecount += 1

        zform = self.zcontext.form("dial", get_child_data=True).update()
        self.assertTrue(zform.has_received())

    def test_pie_chart(self):
        # Example chart
        zchart = self.zcontext.chart('Pie chart', domain=[0.0, 0.0], range=[0.0, 10.0])

        metric_1 = zchart.metric(label='Metric 1')
        metric_2 = zchart.metric(label='Metric 2')
        metric_3 = zchart.metric(label='Metric 3')
        zchart.set_type('Pie')
        zchart.position(2.5, 2.5, 5)

        # Main loop
        while self.framecount < 5:
            data_points_1 = [x * 1.2 for x in randn(10).tolist()]
            data_points_2 = [x * 3.2 for x in randn(10).tolist()]
            data_points_3 = [x * 3.2 for x in randn(10).tolist()]
            metric_1.append_values(y=data_points_1)
            metric_2.append_values(y=data_points_2)
            metric_3.append_values(y=data_points_3)
            zchart.include_metrics([metric_1, metric_2, metric_3])
            zchart.update()
            time.sleep(1.0 / self.target_updates_per_second)
            self.framecount += 1

        zform = self.zcontext.form("pie_chart", get_child_data=True).update()
        self.assertTrue(zform.has_received())
