# mat zoo module

import numpy as np

# ================================
def set_layer(A, S, k=0):
  n = np.shape(A)[0]
  p = n-2*k
  m = 4*(p-1)
  fgByI = False
  
  ii = jj = [k,n-1-k]
  q = np.array(range(p-1))
  r = q
  mChain = 0 
  for i in range(2):
    for j in range(2):
      y = mChain*(p-1) + q
      x = S[y]
   
      fgRev = (i>0 and j==0)
      if fgRev:
        r = - q 
        
      if fgByI:
        i1 = ii[i]+r
        A[i1,jj[j]] = x
      else:
        j1 = jj[j]+r
        A[ii[i],j1] = x.T
        
      mChain += 1      
      fgByI = not (fgByI)
      
    jj = np.flipud(jj)  
    
  return A
  
# ================================

def get_layer(A, k=0):
  n = np.shape(A)[0]
  p = n-2*k
  m = 4*(p-1)
  # S = np.zeros((m, 1), dtype=int )
  S = np.array([], dtype=int)
  fgByI = False
  
  ii = jj = [k,n-1-k]
  q = np.array(range(p-1))
  
  r = q
  x = np.zeros((p-1, 1), dtype=int )
  for i in range(2):
    for j in range(2):
      # print(i, j, ":")
    
      fgRev = (i>0 and j==0)
      if fgRev:
        r = - q ## np.flipud(q)
        # print(r)
        
      if fgByI:
        i1 = ii[i]+r
        x = A[i1,jj[j]]
      else:
        j1 = jj[j]+r
        x = A[ii[i],j1].T
        
      # print(x)  
        
      # print(S.shape, " <> ", x.shape)  
      S = np.concatenate((S,x))
      
      fgByI = not (fgByI)
      
    jj = np.flipud(jj)  
    
  return S

# ================================

def rotate(A, nDT=1):

  D = A
  n = np.shape(A)[0]
  m = int(n/2)
  for k in range(3):

    #print(">>>")
    B = get_layer(A,k)
    #print(B)

    #print("<<<")
    mDT = 3 - k
    C = np.roll(B, mDT*nDT)
    #print(C)

    D = set_layer(D,C,k)

  return D    

# ================================

"""
Creating a Magic Square
=======================

There are three cases: 
  odd, divisible by 4, and 
  even but not divisible by 4 - this one being the most complicated.

"""

def magic(n):
  n = int(n)
  if n < 3:
    raise ValueError("Size must be at least 3")
  if n % 2 == 1:
    p = np.arange(1, n+1)
    return n*np.mod(p[:, None] + p - (n+3)//2, n) + np.mod(p[:, None] + 2*p-2, n) + 1
  elif n % 4 == 0:
    J = np.mod(np.arange(1, n+1), 4) // 2
    K = J[:, None] == J
    M = np.arange(1, n*n+1, n)[:, None] + np.arange(n)
    M[K] = n*n + 1 - M[K]
  else:
    p = n//2
    M = magic(p)
    M = np.block([[M, M+2*p*p], [M+3*p*p, M+p*p]])
    i = np.arange(p)
    k = (n-2)//4
    j = np.concatenate((np.arange(k), np.arange(n-k+1, n)))
    M[np.ix_(np.concatenate((i, i+p)), j)] = M[np.ix_(np.concatenate((i+p, i)), j)]
    M[np.ix_([k, k+p], [0, k])] = M[np.ix_([k+p, k], [0, k])]
  return M 

"""
Creating a spiral Square matrix
=======================
 
A spiral array is a square arrangement of the first   N^2   natural numbers,   
where the
numbers increase sequentially as you go around the edges of the array 
spiraling inwards.

"""  
  
def rot_right(a):
    return zip(*a[::-1])
 
def sp(m, n, start = 0):
  if n == 0:
    yield ()
  else:
    yield tuple(range(start, m + start))
    for row in rot_right(list(sp(n - 1, m, m + start))):
      yield row

def spiral(m):
  S = np.zeros((m, m), dtype=int )
  i = 0
  for row in sp(m, m):
    # print(*row)
    z = list(row) ## [r for r in row]
    # print(z) 
    # S[i,:] = z
    S[i] = z
    i += 1
  
  # print("[final]i=",i)  
  return S  
  